#include "fivewin.ch"
//----------------------------------------------------------------------------//
CLASS TSENDMail

   DATA lShowResult
   DATA amsgerr
   DATA cmsgerr

   DATA csmtp
   DATA cmsgbody
   DATA csubject
   DATA cfrom
   DATA cto
   DATA ccc
   DATA ccomment
   DATA cbinfile
   DATA cpriority
   DATA astruct
   DATA nerror
   DATA nGMT

   METHOD New()
   METHOD Send()
   METHOD CheckBinFile()
   METHOD LogOut() INLINE ::nError := SMTP_LOGOUT()
   METHOD FormatAddress()

ENDCLASS

//----------------------------------------------------------------------------//
METHOD new() CLASS TSENDMail

   ::nerror   := -9999
   ::cSmtp    := ""
   ::cMsgBody := ""
   ::cSubject := "NO SUBJECT"
   ::cfrom    := ""
   ::cto      := ""
   ::ccc      := ""
   ::ccomment := ""
   ::cbinfile := ""
   ::ngmt     := 0
   ::amsgerr := {;
     {-1,   "Cannot create socket" },;
     {-220, "Server not responding"},;
     {-250, "Server not responding"},;
     { 550, "Server not relay"}}

   ::cmsgerr := "Mail delivered"

return self

//----------------------------------------------------------------------------//
METHOD send() CLASS TSENDMail

   LOCAL _nerr

   if !empty( ::csmtp )

      ::astruct := array(19)

      if ::lshowresult == nil
         ::lshowresult := .t.
      endif

      ::cFrom    := alltrim( ::cFrom )
      ::cTo      := alltrim( ::cTo )
      ::cCc      := alltrim( ::cCc )
      ::cSubject := alltrim( ::cSubject )
      ::cmsgbody := alltrim( ::cmsgbody )
      ::cmsgbody := strtran( ::cmsgbody, CHR(13)+CHR(10)+"."+CHR(13)+CHR(10),"" )

      // priority
      if valtype( ::cpriority ) == "N"
         ::cpriority := ltrim(str(::cpriority))
      else
         ::cpriority := "3"
      endif

      ::ngmt *= 3600
      ::ngmt := if(::ngmt>0,"-","+") + ltrim(str(::ngmt))

      ::astruct[ 1] := alltrim(::csmtp)
      ::astruct[ 2] := ::cmsgbody
      ::astruct[ 3] := ::csubject

      ::FormatAddress()
      ::astruct[10] := ""
      ::astruct[11] := ""

      ::checkbinfile()

      ::astruct[12] := ""
      ::astruct[13] := ""
      ::astruct[14] := ::ccomment
      ::astruct[15] := ""
      ::astruct[16] := ::cbinfile
      ::astruct[17] := "0,debug.log,1,1,1,1,"+::ngmt+",1,25,0"
      ::astruct[18] := ""
      ::astruct[19] := ""

      if ( ::nerror := send_maila( ::astruct, 0, 0 ) ) == 0
         ::logout()
      else
         if ( _nerr := ascan( ::amsgerr, { |e| e[1] == ::nerror }) ) > 0
            ::cmsgerr := ::amsgerr[_nerr][2]
         else
            ::cmsgerr := "Error " + ltrim(str(::nerror))
         endif
      endif

   endif

   if ::lShowResult
      msginfo( ::cmsgerr )
   endif

   return self

//----------------------------------------------------------------------//
METHOD CheckBinFile() CLASS TSENDMail

   LOCAL afiles, i, j := 0

   if valtype( ::cbinfile ) != "C"
      ::cbinfile := ""
   else
      ::cbinfile := alltrim( ::cbinfile )
      afiles     := str2arrex( ::cbinfile , "," )
      ::cbinfile := ""
      if ( j := len( afiles ) ) > 0
         for i := 1 to j
            if file( afiles[i] )
               ::cbinfile += afiles[i] + if( i < j, ",", "" )
            endif
         next
      endif
   endif

   return self

//----------------------------------------------------------------------//
METHOD FormatAddress( nFormat ) CLASS TSENDMail

   LOCAL nAt, i, j, k
   LOCAL aTemp := {}
   LOCAL cTemp
   LOCAL cTo := ""
   LOCAL cToFullName := ""
   LOCAL cLegal
   LOCAL aIndex  := {{4,5},{6,7},{8,9}}
   LOCAL aFormat := {::cFrom,::cTo,::cCC}
   LOCAL ii

   FOR ii := 1 TO 3

      aTemp := {}
      cTemp := aFormat[ii]
      cTo   := ""
      cToFullName := ""

      if ( nAt := At( ",", cTemp ) ) > 0
         While ( nAt := At( ",", cTemp ) ) > 0
            AAdd( aTemp, alltrim( left( ctemp, nAt - 1 ) ) )
            cTemp := substr( cTemp, nAt + 1 )
         Enddo
         AAdd( aTemp, alltrim( cTemp ) )
      else
         if !empty( cTemp )
            aTemp := { cTemp }
         endif
      endif

      if ( k := len( aTemp ) ) > 0
        for i := 1 to k
          if "@" $ aTemp[i]
            if "<" $ aTemp[i] .AND. ">" $ aTemp[i]
              nAt := At( "<", aTemp[i] )
              cToFullName += aTemp[i] + if ( i < k, ",", "" )
              cLegal := substr( aTemp[i], nAt + 1 )
              cLegal := strtran( cLegal, ">", "" )
              cTo += cLegal + if ( i < k, ",", "" )
            else
              cToFullName += aTemp[i] + if ( i < k, ",", "" )
              cTo += aTemp[i] + if ( i < k, ",", "" )
            endif
          endif
        next
      endif

      ::astruct[aIndex[ii][1]] := cTo
      ::astruct[aIndex[ii][2]] := cToFullName

   NEXT ii

   return self

//----------------------------------------------------------------------------//
STATIC FUNCTION STR2ARREX( CSAMPLE, CSEP )

   local nat, aret := {}, cret

   if empty( csample )
      return {}
   endif

   if (nat := at(csep,csample )) == 0
      return { alltrim( csample ) }
   endif

   while ( nat := at(csep,csample) ) > 0
      cret := alltrim( left( csample, nat - 1 ) )
      aadd( aret, cret )
      csample := substr( csample, nat + 1 )
   enddo
   aadd( aret, alltrim( csample ) )

   return aret
